#!/bin/sh
# Copyright (c) 2019 Qualcomm Technologies, Inc.
# All Rights Reserved.
# Confidential and Proprietary - Qualcomm Technologies, Inc.

MAPLITEMODE_DEBUG_OUTOUT=0
chirp_count=0
search_hyd_restart=0
sta_iface=""
dpp_lite_sta_connector=0
bhBSS_lite=""
map_num_radio=0
config_get_bool ezmesh repacd 'Ezmesh' '0'
    if [ "$ezmesh" -eq 1 ]; then
        MAP='ezmesh'
    else
        MAP='hyd'
    fi

# Include to use vlan monitoring functionality
. /lib/functions/repacd-vlanmon-map.sh

# Emit a message at debug level.
# input: $1 - the message to log
__repacd_maplitemode_debug() {
    local stderr=''
    if [ "$MAPLITEMODE_DEBUG_OUTOUT" -gt 0 ]; then
        stderr='-s'
    fi

    logger $stderr -t repacd.maplitemode -p user.debug "$1"
}

__repacd_maplite_chirp() {
    local mod_value
    local staBitRate=$(repacdcli $sta_iface get_bitrate)
    local connectorFound

    if [ "$staBitRate" -eq 0 -o -z "$staBitRate" ]; then
        __repacd_maplitemode_debug "Sta Bit Rate Invalid"
        search_hyd_restart=0
        if [ "$chirp_count" -eq 0 ]; then
            wpa_cli -p /var/run/wpa_supplicant-$sta_iface dpp_configurator_remove 1
            wpa_cli -p /var/run/wpa_supplicant-$sta_iface dpp_bootstrap_gen type=qrcode
            wpa_cli -p /var/run/wpa_supplicant-$sta_iface dpp_bootstrap_info 1
            wpa_cli -p /var/run/wpa_supplicant-$sta_iface dpp_bootstrap_get_uri 1
        fi

        chirp_count=$((chirp_count + 1))
        mod_value=$((chirp_count % 8))

        # Send 2 chirps quickly
        if [ "$chirp_count" -eq 1 -o "$chirp_count" -eq 2 ]; then
            wpa_cli -p /var/run/wpa_supplicant-$sta_iface dpp_stop_listen
            wpa_cli -p /var/run/wpa_supplicant-$sta_iface dpp_chirp own=1
        fi

        if [ "$mod_value" -ne 0 ]; then
           return
        fi

        connectorFound=$(cat "/tmp/map_sta_info.tmp" | grep DPP_STA_CONNECTOR)
        __repacd_maplitemode_debug "$mod_value"
        __repacd_maplitemode_debug "Connector found file : $connectorFound"
        __repacd_maplitemode_debug "Connector found UCI : $dpp_lite_sta_connector"

        if [ -n "$connectorFound" ]; then
            __repacd_maplitemode_debug "Conf Object received . Dont chirp"
           return
        fi

        if [ "$dpp_lite_sta_connector" -ne 0 ]; then
            __repacd_maplitemode_debug "Conf Object received . Dont chirp"
           return
        fi

        __repacd_maplitemode_debug "Start Chirping on $sta_iface , count $chirp_count"
        wpa_cli -p /var/run/wpa_supplicant-$sta_iface dpp_stop_listen
        wpa_cli -p /var/run/wpa_supplicant-$sta_iface dpp_chirp own=1
    else
        __repacd_maplitemode_debug "Valid STA BitRate $staBitRate"
        dpp_lite_sta_connector=0
        if [ "$search_hyd_restart" -eq 0 ]; then
            search_hyd_restart=1
            __repacd_maplitemode_debug "Restart HYD to send search"
            /etc/init.d/$MAP restart
        fi
    fi
}

__repacd_maplite_get_sta_iface() {
    local config="$1"
    local iface network disabled device

    config_get iface "$config" ifname
    config_get network "$config" network
    config_get disabled "$config" disabled '0'
    config_get mapVlanID "$config" mapVlanID '0'
    config_get mode "$config" mode
    config_get MapBSSType "$config" MapBSSType '0'

    if [ -n "$iface" -a "$disabled" -eq 0 -a "$mode" = "sta" ]; then
        sta_iface=$iface
        config_get dpp_lite_sta_connector "$config" dpp_connector
    fi
}

__repacd_maplite_program_wds() {
    local port_lan port_lan1 port
    local dut_stamac mac_addr
    local i add_entry local_entry

    __repacd_maplitemode_debug "Maplite program wds for BH $bhBSS_lite"

    if [ -n "$bhBSS_lite" ]; then
        port_lan=$(brctl showstp br-lan | grep $bhBSS_lite | awk -F "[()]" '{print $2}' )
        port_lan1=$(brctl showstp br-lan1 | grep $bhBSS_lite | awk -F "[()]" '{print $2}' )
        dut_stamac=$(wlanconfig $bhBSS_lite list | grep -o -E \
                                                        '([[:xdigit:]]{1,2}:){5}[[:xdigit:]]{1,2}')
        __repacd_maplitemode_debug "port_lan=$port_lan , port_lan1=$port_lan1"

        brctl showmacs br-lan | \
        while read i
        do
            port=$(echo $i | cut -d ' ' -f1)
            local_entry=$(echo $i | cut -d ' ' -f3)
            if [ "$port" == "$port_lan" -a "$local_entry" = "no" ]; then
                mac_addr=`echo $i | grep -o -E '([[:xdigit:]]{1,2}:){5}[[:xdigit:]]{1,2}'`
                add_entry=$(wlanconfig $bhBSS_lite hmwds read-table | grep $mac_addr)
                if [ "$dut_stamac" -eq "$mac_addr" ]; then
                    continue
                fi

                if [ -z "$add_entry" ]; then
                    __repacd_maplitemode_debug "Add hmwds entry $bhBSS_lite $dut_stamac $mac_addr"
                    wlanconfig $bhBSS_lite hmwds add-addr $dut_stamac $mac_addr
                    sleep 1
                fi
            fi
        done

        brctl showmacs br-lan1 | \
        while read i
        do
            port=$(echo $i | cut -d ' ' -f1)
            local_entry=$(echo $i | cut -d ' ' -f3)
            if [ "$port" == "$port_lan1" -a "$local_entry" = "no" ]; then
                mac_addr=`echo $i | grep -o -E '([[:xdigit:]]{1,2}:){5}[[:xdigit:]]{1,2}'`
                add_entry=$(wlanconfig $bhBSS_lite hmwds read-table | grep $mac_addr)
                if [ -z "$add_entry" ]; then
                    __repacd_maplitemode_debug "Add hmwds entry $bhBSS_lite $dut_stamac $mac_addr"
                    wlanconfig $bhBSS_lite hmwds add-addr $dut_stamac $mac_addr
                    sleep 1
                fi
            fi
        done
    fi
}

__repacd_maplite_program_peer_isolation() {
    local port_lan port_lan1 port
    local dut_stamac mac_addr
    local i add_entry local_entry

    __repacd_maplitemode_debug "Maplite program peer isolation for sta $sta_iface"

    if [ -n "$sta_iface" ]; then
        port_lan=$(brctl showstp br-lan | grep $sta_iface | awk -F "[()]" '{print $2}' )
        __repacd_maplitemode_debug "port_lan=$port_lan"

        brctl showmacs br-lan | \
        while read i
        do
            port=$(echo $i | cut -d ' ' -f1)
            local_entry=$(echo $i | cut -d ' ' -f3)
            if [ "$port" == "$port_lan" -a "$local_entry" = "no" ]; then
                mac_addr=`echo $i | grep -o -E '([[:xdigit:]]{1,2}:){5}[[:xdigit:]]{1,2}'`
                add_entry=$(wlanconfig $sta_iface peer_isolation list | grep $mac_addr)

                if [ -z "$add_entry" ]; then
                    __repacd_maplitemode_debug "Add peer isolation entry $mac_addr"
                    wlanconfig $sta_iface peer_isolation add $mac_addr
                    sleep 1
                fi
            fi
        done
    fi
}

__repacd_maplite_get_bhBSS() {
    local config="$1"
    local iface network disabled device MapBSSType

    config_get iface "$config" ifname
    config_get disabled "$config" disabled '0'
    config_get mode "$config" mode
    config_get MapBSSType "$config" MapBSSType '0'

    if [ -n "$iface" -a "$disabled" -eq 0 -a "$mode" = "ap" ]; then
        if [ $((MapBSSType & 0x40)) -eq 64 ]; then
            bhBSS_lite=$iface
        fi
    fi
}

# Enable Radio
#
# input: $1 config: section to update
__repacd_config_get_num_radio() {
    local config="$1"
    local country="$2"
    config_get hwmode "$config" hwmode
    config_get type "$config" type

    if [ "$hwmode" = '11ad' ] && [ "$type" = 'mac80211' ]; then
        return
    fi

    map_num_radio=$((map_num_radio + 1))
}

repacd_config_set_channel() {
    local channelSet
    local channelCurrent
    local bitRate=$(repacdcli $sta_iface get_bitrate)
    if [ "$bitRate" -eq 0 -o -z "$bitRate" ]; then
        __repacd_maplitemode_debug "Invalid STA Bit rate. Dont set channel"
        return
    fi

    __repacd_maplitemode_debug "Map num radio $map_num_radio"
    for i in 0 1 2 3 4 5; do
        if [ "$i" -lt "$map_num_radio" ]; then
            channelSet=$(uci show wireless | grep wifi$i | grep channel | cut -d '=' -f2)
            channelCurrent=$(iwlist ath$i channel | grep Current | awk '{print $5}' | cut -d ')' -f1)
            eval channelSet=$channelSet
            __repacd_maplitemode_debug "Set channel $channelSet on wifi$i"
            __repacd_maplitemode_debug "Current channel $channelCurrent on wifi$i"
            if [ "$channelSet" -ne "$channelCurrent" ]; then
                iwconfig ath$i channel $channelSet
            fi
        fi
    done
}

repacd_maplitemode_init() {
    __repacd_maplitemode_debug "Map Lite Mode Init"
    repacd_map_vlanmon_init
    chirp_count=0

    # Get number of radio
    config_load wireless
    config_foreach __repacd_config_get_num_radio wifi-device
}

repacd_maplitemode_check() {
    local restartWifi bh_type onboarding_type

    # First resolve the config parameters.
    config_load repacd
    config_get_bool restartWifi MAPConfig 'restartWifiDPP' '0'
    config_get bh_type MAPConfig 'MapBackaulType'
    config_get onboarding_type MAPConfig 'OnboardingType'

    # Set channel
    repacd_config_set_channel

    config_load wireless
    config_foreach __repacd_maplite_get_bhBSS wifi-iface

    __repacd_maplitemode_debug "Map Lite Mode Checks"
    repacd_map_vlanmon_check

    __repacd_maplite_program_peer_isolation

    if [ "$bh_type" = "wifi" -a "$onboarding_type" = "dpp" ]; then
        config_load wireless
        config_foreach __repacd_maplite_get_sta_iface wifi-iface

        __repacd_maplite_chirp
    fi

    if [ "$restartWifi" -eq 1 ]; then
        uci set repacd.MAPConfig.restartWifiDPP='0'
        uci commit repacd

        wifi load
    fi
}
